@@ -142,7 +142,7 @@ public class NFABuilder implements NinaAction {
 
 	//
 	private NFAObject<Object, NFAState, Void> createQuoteNFA1(
-			NinaEvent q, CharSequence s, int qf) {
+			NinaEvent q, CharSequence s) {
 		List<NFAObject<Object, NFAState, Void>> l;
 		NFAObject<Object, NFAState, Void> o;
 		Integer c;
@@ -158,6 +158,34 @@ public class NFABuilder implements NinaAction {
 		return NFAConcatenation.newInstance(l);
 	}
 
+	//
+	private NFAObject<Object, NFAState, Void> createQuoteNFA2(
+			NinaEvent q, CharSequence s) {
+		List<NFAObject<Object, NFAState, Void>> l;
+		NFAObject<Object, NFAState, Void> o, p;
+		Integer c;
+		char x, X;
+
+		l = new ArrayList<NFAObject<Object, NFAState, Void>>();
+		for(int k = 0; k < s.length(); k++) {
+			x = s.charAt(k);
+			X = Character.toUpperCase(x);
+			x = Character.toLowerCase(x);
+			if(x == X) {
+				c = Nina.prendCharcode(q.getCharset(), x);
+				o = SingleObjectNFA.newInstance((Object)c);
+				l.add(o);
+			} else {
+				c = Nina.prendCharcode(q.getCharset(), x);
+				o = SingleObjectNFA.newInstance((Object)c);
+				c = Nina.prendCharcode(q.getCharset(), X);
+				p = SingleObjectNFA.newInstance((Object)c);
+				l.add(NFAAlternative.newInstance(o, p));
+			}
+		}
+		return NFAConcatenation.newInstance(l);
+	}
+
 	//
 	private NFAObject<Object, NFAState, Void> createQuoteNFA(
 			NinaEvent q, CharSequence s, int qf) {
@@ -167,9 +195,11 @@ public class NFABuilder implements NinaAction {
 		Matcher m;
 
 		if(qf == 'I') {
-			o = createQuoteNFA1(q, s.toString().toUpperCase(), qf);
-			p = createQuoteNFA1(q, s.toString().toLowerCase(), qf);
+			o = createQuoteNFA1(q, s.toString().toUpperCase());
+			p = createQuoteNFA1(q, s.toString().toLowerCase());
 			return NFAAlternative.newInstance(o, p);
+		} else if(qf == 'i') {
+			return createQuoteNFA2(q, s);
 		} else if(qf == 'n') {
 			l = new ArrayList<NFAObject<Object, NFAState, Void>>();
 			m = CHARX.matcher(s);
@@ -177,13 +207,13 @@ public class NFABuilder implements NinaAction {
 				x  = m.group();
 				nc = Normalizer.normalize(x, Normalizer.Form.NFC);
 				nd = Normalizer.normalize(x, Normalizer.Form.NFD);
-				o  = createQuoteNFA1(q, nc, qf);
-				p  = createQuoteNFA1(q, nd, qf);
+				o  = createQuoteNFA1(q, nc);
+				p  = createQuoteNFA1(q, nd);
 				l.add(NFAAlternative.newInstance(o, p));
 			}
 			return NFAConcatenation.newInstance(l);
 		} else {
-			return createQuoteNFA1(q, s, qf);
+			return createQuoteNFA1(q, s);
 		}
 	}
 
@@ -577,4 +607,46 @@ public class NFABuilder implements NinaAction {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeStringSequence(net.morilib.nina.NinaEvent, int, java.lang.String)
+	 */
+	@Override
+	public void setEdgeStringSequence(NinaEvent q, int l, String cs) {
+		StringBuilder b;
+
+		if(l < 0) {
+			_setRe(q, "[" + cs + "]*");
+		} else {
+			b = new StringBuilder();
+			for(int i = 0; i < l; i++) {
+				b.append("[").append(cs).append("]");
+			}
+			_setRe(q, b.toString());
+		}
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeFormatInteger(net.morilib.nina.NinaEvent, int, java.lang.String)
+	 */
+	@Override
+	public void setEdgeFormatInteger(NinaEvent q, int l, String flg) {
+		setEdgeStringSequence(q, l, "0-9");
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeFormatHex(net.morilib.nina.NinaEvent, int, java.lang.String)
+	 */
+	@Override
+	public void setEdgeFormatHex(NinaEvent q, int l, String flg) {
+		setEdgeStringSequence(q, l, "0-9A-Fa-f");
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeFormatOct(net.morilib.nina.NinaEvent, int, java.lang.String)
+	 */
+	@Override
+	public void setEdgeFormatOct(NinaEvent q, int l, String flg) {
+		setEdgeStringSequence(q, l, "0-7");
+	}
+
 }
