@@ -17,6 +17,8 @@
 package net.morilib.regex.dfa;
 
 import java.io.IOException;
+import java.util.ArrayDeque;
+import java.util.Deque;
 
 import net.morilib.automata.NFAState;
 import net.morilib.automata.nfa.NFAAccept;
@@ -30,9 +32,12 @@ import net.morilib.automata.nfa.NFAs;
 import net.morilib.automata.nfa.RegexParseException;
 import net.morilib.automata.nfa.SingleObjectNFA;
 import net.morilib.automata.nfa.SingleSetNFA;
+import net.morilib.range.CharSetException;
 import net.morilib.range.CharSets;
 import net.morilib.range.IntervalsInt;
 import net.morilib.range.Range;
+import net.morilib.range.integer.IntCharSets;
+import net.morilib.range.integer.IntRange;
 import net.morilib.unicode.UnicodeUtils;
 
 /**
@@ -100,20 +105,10 @@ public class RegexParser {
 		_l.add(x);
 	}
 
-	private int __lookahead_state;
-	private int __lookahead_mark = -1;
-	private int[] __lookahead = null;
-	private int __lookahead_ptr = -1;
-	private int[] __lookaheadw = null;
-	private int __lookaheadw_ptr = -1;
-	private boolean __lookahead_ok = true;
 
 	private int _unreadl = -1;
 
 	void INCLUDE(java.io.Reader rd) {
-		if(__lookahead_ptr >= 0) {
-			throw new IllegalStateException();
-		}
 		streamStack.push(rd);
 	}
 
@@ -161,15 +156,6 @@ public class RegexParser {
 				c = unread;
 				unread = -1;
 				__logprint("Read unread: ", c);
-			} else if(__lookahead_ptr >= 0) {
-				if(__lookahead_ptr < __lookahead.length) {
-					c = __lookahead[__lookahead_ptr++];
-				} else {
-					__lookahead = null;
-					__lookahead_ptr = -1;
-					c = _read();
-				}
-				__logprint("Read Lookahead: ", c);
 			} else if((c = _read1()) != -1) {
 				__logprint("Read: ", c);
 			} else {
@@ -207,76 +193,8 @@ public class RegexParser {
 	private void __puttrace() {
 	}
 
-	void LOOKAHEAD(int c) {
-		int[] a;
-
-		if(__lookaheadw == null) {
-			__lookahead_state = STATE;
-			__lookaheadw = new int[72];
-			__lookaheadw_ptr = 0;
-			__lookaheadw[__lookaheadw_ptr++] = c;
-		} else if(__lookaheadw_ptr < __lookaheadw.length) {
-			__lookaheadw[__lookaheadw_ptr++] = c;
-		} else {
-			a = new int[__lookaheadw.length * 2];
-			System.arraycopy(__lookaheadw, 0, a, 0,
-					__lookaheadw.length);
-			__lookaheadw = a;
-			__lookaheadw[__lookaheadw_ptr++] = c;
-		}
-	}
-
-	private void __copy_lookahead(int p) {
-		int[] a;
-
-		if(__lookahead_ptr > 0) {
-			a = new int[__lookahead.length - __lookahead_ptr];
-			System.arraycopy(__lookahead, __lookahead_ptr, a, 0, a.length);
-			__lookahead = a;
-		}
-
-		if(__lookahead == null) {
-			a = new int[__lookaheadw_ptr];
-		} else if(__lookaheadw_ptr < __lookahead.length) {
-			a = __lookahead;
-		} else {
-			a = new int[__lookaheadw_ptr];
-		}
-		System.arraycopy(__lookaheadw, 0, a, 0, __lookaheadw_ptr);
-		__lookahead = a;
-		__lookahead_ptr = p;
-		__lookaheadw = null;
-		__lookaheadw_ptr = -1;
-	}
-
-	void LOOKAHEAD_COMMIT() {
-		if(__lookahead_mark < 0) {
-			__lookaheadw = null;
-			__lookaheadw_ptr = -1;
-		} else {
-			__copy_lookahead(__lookahead_mark);
-		}
-		__lookahead_mark = -1;
-		__logprint("Commit Lookahead");
-	}
-
-	void LOOKAHEAD_RB() {
-		__copy_lookahead(0);
-		STATE = __lookahead_state;
-		__lookahead_ok = false;
-		__lookahead_mark = -1;
-		__logprint("Rollback Lookahead");
-	}
-
-	void LOOKAHEAD_MARK() {
-		__lookahead_mark = __lookaheadw_ptr;
-	}
-
 
 	private int elem_step(int  $c)  throws java.io.IOException {
-		boolean __l__ = __lookahead_ok;
-
-		__lookahead_ok = true;
 		switch(STATE) {
 		case 0:
 			if($c < 0) {
@@ -288,16 +206,16 @@ public class RegexParser {
 				return 1;
 			}
 		case 1:
-			if(($c == '(')) {
+			if(($c == '[')) {
 				STATE = 2;
 				return 1;
-			} else if(($c == '.')) {
+			} else if(($c == '\\')) {
 				STATE = 3;
 				return 1;
-			} else if(($c == '[')) {
+			} else if(($c == '.')) {
 				STATE = 4;
 				return 1;
-			} else if(($c == '\\')) {
+			} else if(($c == '(')) {
 				STATE = 5;
 				return 1;
 			} else if($c >= 0) {
@@ -308,156 +226,198 @@ public class RegexParser {
 		case 6:
 			return 0;
 		case 5:
-			if(($c == 'u')) {
+			if(($c == '?')) {
 				STATE = 7;
 				return 1;
-			} else if(($c == 'D') || ($c == 'S') || ($c == 'W') || ($c == 'X') || ($c == 'd') || ($c == 's') || ($c == 'w')) {
+			} else if($c < 0) {
 				STATE = 8;
 				return 1;
-			} else if(($c == 'x')) {
-				STATE = 9;
+			} else if($c >= 0) {
+				UNGET($c);
+				STATE = 8;
 				return 1;
-			} else if(($c == 'P') || ($c == 'p')) {
+			}
+		case 8:
+			if($c >= 0) {
+				__stkpush(9, ENGINE_regexParser);
+				STATE = 0;
+				return NINA_ACCEPT;
+			}
+			return 0;
+		case 9:
+			if(($c == ')')) {
 				STATE = 10;
 				return 1;
 			}
 			return 0;
 		case 10:
-			if(($c == '{')) {
+			return 0;
+		case 7:
+			if(($c == ':')) {
+				STATE = 8;
+				return 1;
+			} else if(($c == ')')) {
 				STATE = 11;
 				return 1;
-			}
-			return 0;
-		case 11:
-			if(($c == '}')) {
-				STATE = 12;
-				return 1;
 			} else if($c >= 0) {
-				STATE = 13;
-				return 1;
-			}
-			return 0;
-		case 13:
-			if(($c == '}')) {
 				STATE = 12;
 				return 1;
-			} else if($c >= 0) {
-				STATE = 13;
-				return 1;
 			}
 			return 0;
 		case 12:
-			return 0;
-		case 9:
-			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
-				$buffer = new StringBuffer();$buffer.append((char)$c);
-				STATE = 14;
+			if(($c == ':')) {
+				STATE = 8;
 				return 1;
-			}
-			return 0;
-		case 14:
-			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
-				$buffer.append((char)$c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 15;
-				return 1;
-			} else if($c < 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 15;
+			} else if(($c == ')')) {
+				STATE = 11;
 				return 1;
 			} else if($c >= 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 15;
+				STATE = 12;
 				return 1;
 			}
-		case 15:
 			return 0;
-		case 8:
+		case 11:
+				STATE = 1;
+				return 1;
+		case 4:
 			return 0;
-		case 7:
-			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
-				$buffer = new StringBuffer();$buffer.append((char)$c);
+		case 3:
+			if(($c == 'x')) {
+				STATE = 13;
+				return 1;
+			} else if(($c == 'u')) {
+				STATE = 14;
+				return 1;
+			} else if(($c == 'D') || ($c == 'S') || ($c >= 'W' && $c <= 'X') || ($c == 'd') || ($c == 's') || ($c == 'w')) {
+				STATE = 15;
+				return 1;
+			} else if(($c == 'P') || ($c == 'p')) {
 				STATE = 16;
 				return 1;
+			} else if($c >= 0) {
+				STATE = 6;
+				return 1;
 			}
 			return 0;
 		case 16:
-			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
-				$buffer.append((char)$c);
+			if(($c == '{')) {
 				STATE = 17;
 				return 1;
-			} else if($c < 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 18;
-				return 1;
-			} else if($c >= 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 18;
-				return 1;
 			}
-		case 18:
 			return 0;
 		case 17:
-			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
-				$buffer.append((char)$c);
+			if(($c == '}')) {
+				STATE = 18;
+				return 1;
+			} else if($c >= 0) {
 				STATE = 19;
 				return 1;
-			} else if($c < 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 18;
-				return 1;
-			} else if($c >= 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 18;
-				return 1;
-			}
-		case 19:
-			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
-				$buffer.append((char)$c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 18;
-				return 1;
-			} else if($c < 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 18;
-				return 1;
-			} else if($c >= 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
-				STATE = 18;
-				return 1;
-			}
-		case 4:
-				STATE = 20;
-				return 1;
-		case 20:
-			if($c >= 0) {
-				__stkpush(21, ENGINE_cset);
-				STATE = 0;
-				return NINA_ACCEPT;
 			}
 			return 0;
-		case 21:
-			if(($c == ']')) {
+		case 19:
+			if(($c == '}')) {
+				STATE = 18;
+				return 1;
+			} else if($c >= 0) {
+				STATE = 19;
+				return 1;
+			}
+			return 0;
+		case 18:
+			return 0;
+		case 15:
+			return 0;
+		case 14:
+			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
+				$buffer = new StringBuffer();$buffer.append((char)$c);
+				STATE = 20;
+				return 1;
+			}
+			return 0;
+		case 20:
+			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
+				$buffer.append((char)$c);
+				STATE = 21;
+				return 1;
+			} else if($c < 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 22;
+				return 1;
+			} else if($c >= 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
 				STATE = 22;
 				return 1;
 			}
-			return 0;
 		case 22:
 			return 0;
-		case 3:
-			return 0;
-		case 2:
-			if($c >= 0) {
-				__stkpush(23, ENGINE_regexParser);
-				STATE = 0;
-				return NINA_ACCEPT;
+		case 21:
+			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
+				$buffer.append((char)$c);
+				STATE = 23;
+				return 1;
+			} else if($c < 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 22;
+				return 1;
+			} else if($c >= 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 22;
+				return 1;
 			}
-			return 0;
 		case 23:
-			if(($c == ')')) {
+			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
+				$buffer.append((char)$c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 22;
+				return 1;
+			} else if($c < 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 22;
+				return 1;
+			} else if($c >= 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 22;
+				return 1;
+			}
+		case 13:
+			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
+				$buffer = new StringBuffer();$buffer.append((char)$c);
 				STATE = 24;
 				return 1;
 			}
 			return 0;
 		case 24:
+			if(($c >= '0' && $c <= '9') || ($c >= 'A' && $c <= 'F') || ($c >= 'a' && $c <= 'f')) {
+				$buffer.append((char)$c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 25;
+				return 1;
+			} else if($c < 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 25;
+				return 1;
+			} else if($c >= 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 16);
+				STATE = 25;
+				return 1;
+			}
+		case 25:
+			return 0;
+		case 2:
+				STATE = 26;
+				return 1;
+		case 26:
+			if($c >= 0) {
+				__stkpush(27, ENGINE_cset);
+				STATE = 0;
+				return NINA_ACCEPT;
+			}
+			return 0;
+		case 27:
+			if(($c == ']')) {
+				STATE = 28;
+				return 1;
+			}
+			return 0;
+		case 28:
 			return 0;
 		}
 		return 0;
@@ -465,74 +425,62 @@ public class RegexParser {
 
 	private boolean elem_accepted() {
 		return (STATE == 18 ||
-				STATE == 3 ||
+				STATE == 4 ||
 				STATE == 6 ||
 				STATE == 22 ||
-				STATE == 8 ||
-				STATE == 24 ||
-				STATE == 12 ||
+				STATE == 25 ||
+				STATE == 10 ||
+				STATE == 28 ||
 				STATE == 15);
 	}
 
 	int elem_execaction(int  $c) {
 		switch(STATE) {
-		case 10:
-			// State40   
-			rn = $c;
-			break;
 		case 18:
-			// State35        
-			_ = atom($int);
+			// State42                    
+			_ = prop(rn, b.toString());
 			break;
 		case 21:
 			break;
-		case 4:
-			// State22                 
-			b = new StringBuilder();
+		case 7:
+			// State56   
+			// none
 			break;
-		case 6:
-			// newState17   
-			_ = atom($c);
+		case 10:
+			// newState19   
+			_ = par(_);
+			break;
+		case 13:
+			// State36   
+			// none
 			break;
 		case 22:
-			// State26                
-			_ = cset(b.toString());
-			break;
-		case 14:
-			break;
-		case 15:
-			// State37        
+			// State35        
 			_ = atom($int);
 			break;
-		case 0:
-			break;
 		case 3:
-			// State39   
-			_ = dot();
-			break;
-		case 23:
-			break;
-		case 16:
-			break;
-		case 8:
-			// State38     
-			_ = esc($c);
-			break;
-		case 11:
-			// State43                 
-			b = new StringBuilder();
-			break;
-		case 20:
-			break;
-		case 2:
-			break;
-		case 17:
-			break;
-		case 5:
 			// State33   
 			// none
 			break;
-		case 13:
+		case 24:
+			break;
+		case 23:
+			break;
+		case 5:
+			// State55   
+			nowf = 0;
+			break;
+		case 17:
+			// State43                 
+			b = new StringBuilder();
+			break;
+		case 26:
+			break;
+		case 15:
+			// State38     
+			_ = esc($c);
+			break;
+		case 19:
 			// State41            
 			b.append((char)$c);
 			break;
@@ -540,35 +488,65 @@ public class RegexParser {
 			// newState16   
 			// none
 			break;
-		case 19:
+		case 0:
 			break;
-		case 24:
-			// newState19   
-			_ = par(_);
+		case 8:
 			break;
-		case 9:
-			// State36   
-			// none
+		case 20:
 			break;
-		case 7:
+		case 27:
+			break;
+		case 28:
+			// State26                
+			_ = cset(b.toString());
+			break;
+		case 16:
+			// State40   
+			rn = $c;
+			break;
+		case 14:
 			// State34   
 			// none
 			break;
+		case 4:
+			// State39   
+			_ = dot();
+			break;
 		case 12:
-			// State42                    
-			_ = prop(rn, b.toString());
+			// State57     
+			setflag($c);
+			break;
+		case 9:
+			break;
+		case 2:
+			// State22                 
+			b = new StringBuilder();
+			break;
+		case 25:
+			// State37        
+			_ = atom($int);
+			break;
+		case 6:
+			// newState17   
+			_ = atom($c);
+			break;
+		case 11:
+			// State60   
+			_writef();
 			break;
 		}
 		return 1;
 	}
 
 	boolean elem_isend() {
-		return (STATE == 17 ||
-				STATE == 0 ||
-				STATE == 16 ||
-				STATE == 19 ||
-				STATE == 4 ||
-				STATE == 14);
+		return (STATE == 0 ||
+				STATE == 2 ||
+				STATE == 21 ||
+				STATE == 20 ||
+				STATE == 5 ||
+				STATE == 23 ||
+				STATE == 24 ||
+				STATE == 11);
 	}
 
 	private final Engine ENGINE_elem = new Engine() {
@@ -598,7 +576,7 @@ public class RegexParser {
 		}
 
 		int stateSize() {
-			return 25;
+			return 29;
 		}
 
 		int finallyState() {
@@ -607,17 +585,18 @@ public class RegexParser {
 
 		boolean isDead() {
 		return (STATE == 18 ||
-				STATE == 3 ||
+				STATE == 4 ||
 				STATE == 6 ||
 				STATE == 22 ||
-				STATE == 8 ||
-				STATE == 24 ||
-				STATE == 12 ||
+				STATE == 25 ||
+				STATE == 10 ||
+				STATE == 28 ||
 				STATE == 15);
 		}
 
 		boolean isEmptyTransition() {
-		return (STATE == 4);
+		return (STATE == 2 ||
+				STATE == 11);
 		}
 
 		public String toString() {
@@ -627,9 +606,6 @@ public class RegexParser {
 	};
 
 	private int alternate_step(int  $c)  throws java.io.IOException {
-		boolean __l__ = __lookahead_ok;
-
-		__lookahead_ok = true;
 		switch(STATE) {
 		case 0:
 			if($c < 0) {
@@ -683,23 +659,23 @@ public class RegexParser {
 
 	int alternate_execaction(int  $c) {
 		switch(STATE) {
-		case 0:
-			break;
 		case 6:
 			// newState7                             
 			(__stv[__slen - 1][3]) = _ = alt(((NFAObject<Object, NFAState, Integer>)(__stv[__slen - 1][3])), _);
 			break;
-		case 5:
-			break;
 		case 1:
 			break;
-		case 4:
+		case 5:
+			break;
+		case 2:
 			break;
 		case 3:
 			// firstSel                                      
 			(__stv[__slen - 1][3]) = _;
 			break;
-		case 2:
+		case 4:
+			break;
+		case 0:
 			break;
 		}
 		return 1;
@@ -761,9 +737,6 @@ public class RegexParser {
 	};
 
 	private int cset_step(int  $c)  throws java.io.IOException {
-		boolean __l__ = __lookahead_ok;
-
-		__lookahead_ok = true;
 		switch(STATE) {
 		case 0:
 			if($c < 0) {
@@ -787,7 +760,7 @@ public class RegexParser {
 			if(($c == '[')) {
 				STATE = 2;
 				return 1;
-			} else if(($c >= 0 && $c <= 'Z') || ($c == '\\') || ($c >= '^' && $c <= 2147483647)) {
+			} else if(($c >= ' ' && $c <= 'Z') || ($c == '\\') || ($c >= '^' && $c <= 2147483647)) {
 				STATE = 3;
 				return 1;
 			}
@@ -830,7 +803,19 @@ public class RegexParser {
 		switch(STATE) {
 		case 5:
 			break;
-		case 4:
+		case 3:
+			// State23            
+			b.append((char)$c);
+			break;
+		case 6:
+			// State54            
+			b.append((char)$c);
+			break;
+		case 7:
+			// State24            
+			b.append((char)$c);
+			break;
+		case 0:
 			break;
 		case 1:
 			// State50   
@@ -840,19 +825,7 @@ public class RegexParser {
 			// State52            
 			b.append((char)$c);
 			break;
-		case 3:
-			// State23            
-			b.append((char)$c);
-			break;
-		case 7:
-			// State24            
-			b.append((char)$c);
-			break;
-		case 0:
-			break;
-		case 6:
-			// State54            
-			b.append((char)$c);
+		case 4:
 			break;
 		}
 		return 1;
@@ -914,9 +887,6 @@ public class RegexParser {
 	};
 
 	private int regexParser_step(int  $c)  throws java.io.IOException {
-		boolean __l__ = __lookahead_ok;
-
-		__lookahead_ok = true;
 		switch(STATE) {
 		case 0:
 			if($c < 0) {
@@ -928,88 +898,43 @@ public class RegexParser {
 				return 1;
 			}
 		case 1:
-			if((__l__ && $c == '(')) {
-				LOOKAHEAD($c);
 				STATE = 2;
 				return 1;
-			} else if($c < 0) {
-				STATE = 3;
-				return 1;
-			} else if($c >= 0) {
-				UNGET($c);
-				STATE = 3;
-				return 1;
-			}
-		case 3:
+		case 2:
 			if($c >= 0) {
-				__stkpush(4, ENGINE_alternate);
+				__stkpush(3, ENGINE_alternate);
 				STATE = 0;
 				return NINA_ACCEPT;
 			}
 			return 0;
+		case 3:
+				STATE = 4;
+				return 1;
 		case 4:
-				STATE = 5;
-				return 1;
-		case 5:
-			return 0;
-		case 2:
-			if(($c == '?')) {
-				LOOKAHEAD_COMMIT();
-				STATE = 6;
-				return 1;
-			}
-			return 0;
-		case 6:
-			if(($c == ')')) {
-				STATE = 3;
-				return 1;
-			} else if($c >= 0) {
-				STATE = 7;
-				return 1;
-			}
-			return 0;
-		case 7:
-			if(($c == ')')) {
-				STATE = 3;
-				return 1;
-			} else if($c >= 0) {
-				STATE = 7;
-				return 1;
-			}
 			return 0;
 		}
 		return 0;
 	}
 
 	private boolean regexParser_accepted() {
-		return (STATE == 5);
+		return (STATE == 4);
 	}
 
 	int regexParser_execaction(int  $c) {
 		switch(STATE) {
-		case 3:
-			break;
-		case 7:
-			// State48     
-			setflag($c);
-			break;
-		case 5:
+		case 4:
 			// State45   
 			// none
 			break;
-		case 2:
-			break;
-		case 6:
-			// State47   
-			// none
-			break;
 		case 0:
 			break;
-		case 1:
-			// State46   
-			// none
+		case 3:
 			break;
-		case 4:
+		case 1:
+			// State58   
+			_pushf();
+			break;
+		case 2:
 			break;
 		}
 		return 1;
@@ -1018,7 +943,7 @@ public class RegexParser {
 	boolean regexParser_isend() {
 		return (STATE == 0 ||
 				STATE == 1 ||
-				STATE == 4);
+				STATE == 3);
 	}
 
 	private final Engine ENGINE_regexParser = new Engine() {
@@ -1048,7 +973,7 @@ public class RegexParser {
 		}
 
 		int stateSize() {
-			return 8;
+			return 5;
 		}
 
 		int finallyState() {
@@ -1056,11 +981,12 @@ public class RegexParser {
 		}
 
 		boolean isDead() {
-		return (STATE == 5);
+		return (STATE == 4);
 		}
 
 		boolean isEmptyTransition() {
-		return (STATE == 4);
+		return (STATE == 1 ||
+				STATE == 3);
 		}
 
 		public String toString() {
@@ -1070,9 +996,6 @@ public class RegexParser {
 	};
 
 	private int closure_step(int  $c)  throws java.io.IOException {
-		boolean __l__ = __lookahead_ok;
-
-		__lookahead_ok = true;
 		switch(STATE) {
 		case 0:
 			if($c < 0) {
@@ -1094,13 +1017,13 @@ public class RegexParser {
 				STATE = 3;
 				return 1;
 		case 3:
-			if(($c == '*')) {
+			if(($c == '{')) {
 				STATE = 4;
 				return 1;
 			} else if(($c == '+')) {
 				STATE = 5;
 				return 1;
-			} else if(($c == '{')) {
+			} else if(($c == '*')) {
 				STATE = 6;
 				return 1;
 			} else if(($c == '?')) {
@@ -1111,73 +1034,52 @@ public class RegexParser {
 		case 7:
 			return 0;
 		case 6:
-			if(($c >= '0' && $c <= '9')) {
-				$buffer = new StringBuffer();$buffer.append((char)$c);
+			return 0;
+		case 5:
+			return 0;
+		case 4:
+			if(($c == ',')) {
 				STATE = 8;
 				return 1;
-			} else if(($c == ',')) {
+			} else if(($c >= '0' && $c <= '9')) {
+				$buffer = new StringBuffer();$buffer.append((char)$c);
 				STATE = 9;
 				return 1;
 			}
 			return 0;
 		case 9:
 			if(($c >= '0' && $c <= '9')) {
-				$buffer = new StringBuffer();$buffer.append((char)$c);
+				$buffer.append((char)$c);
 				STATE = 10;
 				return 1;
-			}
-			return 0;
-		case 10:
-			if(($c >= '0' && $c <= '9')) {
-				$buffer.append((char)$c);
+			} else if($c < 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
 				STATE = 11;
 				return 1;
-			} else if($c < 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
 			} else if($c >= 0) {
 				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 11;
+				return 1;
+			}
+		case 11:
+			if(($c == ',')) {
 				STATE = 12;
 				return 1;
 			}
+			return 0;
 		case 12:
-			if(($c == '}')) {
+			if(($c >= '0' && $c <= '9')) {
+				$buffer = new StringBuffer();$buffer.append((char)$c);
 				STATE = 13;
 				return 1;
-			}
-			return 0;
-		case 13:
-			return 0;
-		case 11:
-			if(($c >= '0' && $c <= '9')) {
-				$buffer.append((char)$c);
+			} else if(($c == '}')) {
 				STATE = 14;
 				return 1;
-			} else if($c < 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
-			} else if($c >= 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
 			}
+			return 0;
 		case 14:
-			if(($c >= '0' && $c <= '9')) {
-				$buffer.append((char)$c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
-			} else if($c < 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
-			} else if($c >= 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
-			}
-		case 8:
+			return 0;
+		case 13:
 			if(($c >= '0' && $c <= '9')) {
 				$buffer.append((char)$c);
 				STATE = 15;
@@ -1192,35 +1094,74 @@ public class RegexParser {
 				return 1;
 			}
 		case 16:
-			if(($c == ',')) {
-				STATE = 17;
+			if(($c == '}')) {
+				STATE = 14;
 				return 1;
 			}
 			return 0;
-		case 17:
-			if(($c >= '0' && $c <= '9')) {
-				$buffer = new StringBuffer();$buffer.append((char)$c);
-				STATE = 18;
-				return 1;
-			} else if(($c == '}')) {
-				STATE = 13;
-				return 1;
-			}
-			return 0;
-		case 18:
+		case 15:
 			if(($c >= '0' && $c <= '9')) {
 				$buffer.append((char)$c);
-				STATE = 19;
+				STATE = 17;
 				return 1;
 			} else if($c < 0) {
 				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
+				STATE = 16;
 				return 1;
 			} else if($c >= 0) {
 				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
+				STATE = 16;
 				return 1;
 			}
+		case 17:
+			if(($c >= '0' && $c <= '9')) {
+				$buffer.append((char)$c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 16;
+				return 1;
+			} else if($c < 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 16;
+				return 1;
+			} else if($c >= 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 16;
+				return 1;
+			}
+		case 10:
+			if(($c >= '0' && $c <= '9')) {
+				$buffer.append((char)$c);
+				STATE = 18;
+				return 1;
+			} else if($c < 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 11;
+				return 1;
+			} else if($c >= 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 11;
+				return 1;
+			}
+		case 18:
+			if(($c >= '0' && $c <= '9')) {
+				$buffer.append((char)$c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 11;
+				return 1;
+			} else if($c < 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 11;
+				return 1;
+			} else if($c >= 0) {
+				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
+				STATE = 11;
+				return 1;
+			}
+		case 8:
+			if(($c >= '0' && $c <= '9')) {
+				$buffer = new StringBuffer();$buffer.append((char)$c);
+				STATE = 19;
+				return 1;
+			}
+			return 0;
 		case 19:
 			if(($c >= '0' && $c <= '9')) {
 				$buffer.append((char)$c);
@@ -1228,28 +1169,14 @@ public class RegexParser {
 				return 1;
 			} else if($c < 0) {
 				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
+				STATE = 16;
 				return 1;
 			} else if($c >= 0) {
 				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
+				STATE = 16;
 				return 1;
 			}
 		case 20:
-			if(($c >= '0' && $c <= '9')) {
-				$buffer.append((char)$c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
-			} else if($c < 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
-			} else if($c >= 0) {
-				UNGET($c);$int=Integer.parseInt($buffer.toString(), 10);
-				STATE = 12;
-				return 1;
-			}
-		case 15:
 			if(($c >= '0' && $c <= '9')) {
 				$buffer.append((char)$c);
 				STATE = 21;
@@ -1277,61 +1204,55 @@ public class RegexParser {
 				STATE = 16;
 				return 1;
 			}
-		case 5:
-			return 0;
-		case 4:
-			return 0;
 		}
 		return 0;
 	}
 
 	private boolean closure_accepted() {
 		return (STATE == 3 ||
-				STATE == 4 ||
 				STATE == 5 ||
+				STATE == 6 ||
 				STATE == 7 ||
-				STATE == 13);
+				STATE == 14);
 	}
 
 	int closure_execaction(int  $c) {
 		switch(STATE) {
-		case 8:
-			break;
-		case 13:
-			// State32                        
-			_ = rep(((NFAObject<Object, NFAState, Integer>)(__stv[__slen - 1][3])), rn, $int);
-			break;
-		case 15:
+		case 16:
+			// State31   
+			// none
 			break;
 		case 3:
 			// firstClo                                      
 			(__stv[__slen - 1][3]) = _;
 			break;
-		case 16:
+		case 12:
+			// State30   
+			$int = -1;
+			break;
+		case 13:
+			break;
+		case 10:
+			break;
+		case 11:
 			// State28   
 			rn = $int;
 			break;
-		case 0:
-			break;
-		case 2:
-			break;
-		case 6:
-			// State27   
-			// none
+		case 19:
 			break;
 		case 7:
 			// State21              
 			_ = opt(((NFAObject<Object, NFAState, Integer>)(__stv[__slen - 1][3])));
 			break;
-		case 14:
+		case 0:
 			break;
-		case 21:
+		case 1:
 			break;
-		case 18:
+		case 6:
+			// newState15           
+			_ = clo(((NFAObject<Object, NFAState, Integer>)(__stv[__slen - 1][3])));
 			break;
-		case 19:
-			break;
-		case 9:
+		case 8:
 			// State29   
 			rn = 0;
 			break;
@@ -1339,41 +1260,43 @@ public class RegexParser {
 			// State20               
 			_ = plus(((NFAObject<Object, NFAState, Integer>)(__stv[__slen - 1][3])));
 			break;
-		case 11:
+		case 4:
+			// State27   
+			// none
 			break;
-		case 17:
-			// State30   
-			$int = -1;
+		case 15:
 			break;
-		case 1:
+		case 21:
 			break;
-		case 10:
+		case 18:
 			break;
 		case 20:
 			break;
-		case 12:
-			// State31   
-			// none
+		case 2:
 			break;
-		case 4:
-			// newState15           
-			_ = clo(((NFAObject<Object, NFAState, Integer>)(__stv[__slen - 1][3])));
+		case 9:
+			break;
+		case 14:
+			// State32                        
+			_ = rep(((NFAObject<Object, NFAState, Integer>)(__stv[__slen - 1][3])), rn, $int);
+			break;
+		case 17:
 			break;
 		}
 		return 1;
 	}
 
 	boolean closure_isend() {
-		return (STATE == 0 ||
-				STATE == 19 ||
+		return (STATE == 17 ||
+				STATE == 0 ||
 				STATE == 2 ||
+				STATE == 19 ||
 				STATE == 18 ||
 				STATE == 21 ||
 				STATE == 20 ||
-				STATE == 8 ||
+				STATE == 9 ||
 				STATE == 10 ||
-				STATE == 11 ||
-				STATE == 14 ||
+				STATE == 13 ||
 				STATE == 15);
 	}
 
@@ -1412,10 +1335,10 @@ public class RegexParser {
 		}
 
 		boolean isDead() {
-		return (STATE == 4 ||
-				STATE == 5 ||
+		return (STATE == 5 ||
+				STATE == 6 ||
 				STATE == 7 ||
-				STATE == 13);
+				STATE == 14);
 		}
 
 		boolean isEmptyTransition() {
@@ -1429,9 +1352,6 @@ public class RegexParser {
 	};
 
 	private int concat_step(int  $c)  throws java.io.IOException {
-		boolean __l__ = __lookahead_ok;
-
-		__lookahead_ok = true;
 		switch(STATE) {
 		case 0:
 			if($c < 0) {
@@ -1453,7 +1373,7 @@ public class RegexParser {
 				STATE = 3;
 				return 1;
 		case 3:
-			if(($c >= 0 && $c <= '(') || ($c >= ',' && $c <= '>') || ($c >= '@' && $c <= '{') || ($c >= '}' && $c <= 2147483647)) {
+			if(($c >= ' ' && $c <= '(') || ($c >= ',' && $c <= '>') || ($c >= '@' && $c <= '{') || ($c >= '}' && $c <= 2147483647)) {
 				UNGET($c);
 				STATE = 4;
 				return 1;
@@ -1470,7 +1390,7 @@ public class RegexParser {
 				STATE = 6;
 				return 1;
 		case 6:
-			if(($c >= 0 && $c <= '(') || ($c >= ',' && $c <= '>') || ($c >= '@' && $c <= '{') || ($c >= '}' && $c <= 2147483647)) {
+			if(($c >= ' ' && $c <= '(') || ($c >= ',' && $c <= '>') || ($c >= '@' && $c <= '{') || ($c >= '}' && $c <= 2147483647)) {
 				UNGET($c);
 				STATE = 4;
 				return 1;
@@ -1489,22 +1409,22 @@ public class RegexParser {
 		switch(STATE) {
 		case 5:
 			break;
+		case 1:
+			break;
 		case 0:
 			break;
 		case 3:
 			// firstCat                                      
 			(__stv[__slen - 1][3]) = _;
 			break;
-		case 1:
-			break;
-		case 4:
-			break;
 		case 6:
 			// newState12                            
 			(__stv[__slen - 1][3]) = _ = cat(((NFAObject<Object, NFAState, Integer>)(__stv[__slen - 1][3])), _);
 			break;
 		case 2:
 			break;
+		case 4:
+			break;
 		}
 		return 1;
 	}
@@ -1649,11 +1569,6 @@ public class RegexParser {
 					__logprint("accept " + __stk[__slen - 1]);
 					UNGET(c);
 					st[0] = NINA_ACCEPT;  return -1;
-				} else if(__lookaheadw_ptr >= 0) {
-					__logprint("match failed: try lookahead: ", c);
-					LOOKAHEAD(c);
-					LOOKAHEAD_RB();
-					b = en.accepted();
 				} else if(c == -1) {
 					if(!b)  throw new TokenException();
 					st[0] = NINA_ACCEPT;  return -1;
@@ -1671,10 +1586,6 @@ public class RegexParser {
 				} else if(b) {
 					__logprint("accept " + __stk[__slen - 1]);
 					st[0] = NINA_ACCEPT;  return -1;
-				} else if(__lookaheadw_ptr >= 0) {
-					__logprint("match failed: try lookahead: ", c);
-					LOOKAHEAD_RB();
-					b = en.accepted();
 				} else {
 					__logprint("match failed: ", c);
 					__puttrace();
@@ -1824,11 +1735,65 @@ public class RegexParser {
 	}
 
 	/* @@@-PARSER-CODE-END-@@@ */
+	private static final int DOTALL = 1;
+	private static final int IGNORE_CASE = 2;
+
 	private StringBuilder b;
 	private int rn;
-	private boolean dotAll = false;
-	private boolean ignoreCase = false;
+	private Deque<Integer> flgstk = new ArrayDeque<Integer>();
 	private int caputure = 1;
+	private int nowf = 0;
+	private boolean reset = false;
+
+	private boolean isDotall() {
+		int x;
+
+		x = flgstk.peekFirst();
+		return (x & DOTALL) != 0;
+	}
+
+	private boolean isIgnoreCase() {
+		int x;
+
+		x = flgstk.peekFirst();
+		return (x & IGNORE_CASE) != 0;
+	}
+
+	private void _pushf() {
+		flgstk.addFirst(nowf);
+	}
+
+	private void _writef() {
+		flgstk.removeFirst();
+		flgstk.addFirst(nowf);
+	}
+
+	private void setflag(int c) {
+		switch(c) {
+		case '-':
+			reset = true;
+			break;
+		case 'i':
+			if(reset) {
+				nowf &= ~IGNORE_CASE;
+			} else {
+				nowf |= IGNORE_CASE;
+			}
+			reset = false;
+			break;
+		case 's':
+			if(reset) {
+				nowf &= ~DOTALL;
+			} else {
+				nowf |= DOTALL;
+			}
+			reset = false;
+			break;
+		default:
+			reset = false;
+			break;
+		}
+	}
 
 	private NFAObject<Object, NFAState, Integer> alt(
 			NFAObject<Object, NFAState, Integer> a,
@@ -1859,6 +1824,7 @@ public class RegexParser {
 
 	private NFAObject<Object, NFAState, Integer> par(
 			NFAObject<Object, NFAState, Integer> a) {
+		flgstk.removeFirst();
 		return NFAParenthesis.newInstance(a, caputure++);
 	}
 
@@ -1866,7 +1832,7 @@ public class RegexParser {
 		char u, l;
 		Range o;
 
-		if(ignoreCase) {
+		if(isIgnoreCase()) {
 			u = Character.toUpperCase((char)c);
 			l = Character.toLowerCase((char)c);
 			if(u == l) {
@@ -1882,7 +1848,17 @@ public class RegexParser {
 	}
 
 	private NFAObject<Object, NFAState, Integer> cset(String s) {
-		return SingleSetNFA.newInstance(CharSet.parseRange(s));
+		IntRange r;
+
+		try {
+			r = CharSet.parse(s);
+			if(isIgnoreCase()) {
+				r = IntCharSets.toCaseInsensitive(r);
+			}
+			return SingleSetNFA.newInstance(IntCharSets.toRange(r));
+		} catch(CharSetException e) {
+			throw new TokenException();
+		}
 	}
 
 	private NFAObject<Object, NFAState, Integer> rep(
@@ -1896,7 +1872,7 @@ public class RegexParser {
 
 	private NFAObject<Object, NFAState, Integer> dot() {
 		return SingleSetNFA.newInstance(
-				dotAll ? CharSets.ALL_CHAR : CharSets.DOT);
+				isDotall() ? CharSets.ALL_CHAR : CharSets.DOT);
 	}
 
 	private NFAObject<Object, NFAState, Integer> esc(int c) {
@@ -1921,20 +1897,19 @@ public class RegexParser {
 	}
 
 	private NFAObject<Object, NFAState, Integer> prop(int c, String s) {
-		Range r;
+		IntRange r;
 
-		r = UnicodeUtils.getPropertyRange(s);
+		r = UnicodeUtils.getPropertyIntRange(s);
+//		r = UnicodeUtils.getPropertyRange(s);
 		if(c == 'P') {
-			r = CharSets.complement(r);
+			r = r.complement();
+//			r = CharSets.complement(r);
 		}
-		return SingleSetNFA.newInstance(r);
-	}
 
-	private void setflag(int c) {
-		switch(c) {
-//		case 'i':  ignoreCase = true;  break;
-		case 's':  dotAll = true;  break;
+		if(isIgnoreCase()) {
+			r = IntCharSets.toCaseInsensitive(r);
 		}
+		return SingleSetNFA.newInstance(IntCharSets.toRange(r));
 	}
 
 	/**
@@ -1953,7 +1928,7 @@ public class RegexParser {
 		} catch(TokenException e) {
 			throw new RegexParseException();
 		} catch(IOException e) {
-			throw new RuntimeException();
+			throw new RuntimeException(e);
 		}
 	}
 
