@@ -49,6 +49,8 @@ import net.morilib.nina.cmd.NinatOptions;
 import net.morilib.nina.metalang.NinaMetaLanguage;
 import net.morilib.nina.translate.sh.ShNinatBuiltInCommands;
 import net.morilib.nina.translate.sh.ShNinatFileSystem;
+import net.morilib.range.Interval;
+import net.morilib.range.Range;
 import net.morilib.sh.DefaultShRuntime;
 import net.morilib.sh.ShEnvironment;
 import net.morilib.sh.ShFacade;
@@ -59,6 +61,7 @@ import net.morilib.sh.ShRootShellEnvironment;
 import net.morilib.sh.ShRuntime;
 import net.morilib.sh.ShSyntaxException;
 import net.morilib.sh.misc.XtraceStream;
+import net.morilib.util.Tuple2;
 
 /**
  *
@@ -495,6 +498,38 @@ implements NinaTranslator {
 		out.printf("\t\t\treturn %d;\n", stateSize());
 	}
 
+	//
+	private static int infimumbound(Interval a) {
+		Object o;
+
+		o = a.getInfimumBound();
+		return o == Interval.FROM_INFIMUM ? 0 : ((Integer)o).intValue();
+	}
+
+	//
+	static Map<Object, Range> classifyEdges(
+			DFAState<Object, ?, Void> dfa) {
+		Map<Object, Range> z = new HashMap<Object, Range>();
+		int c;
+
+		for(Interval v : dfa.getAlphabetRanges()) {
+			Tuple2<Object, DFAState<Object, ?, Void>> t;
+			Range g;
+
+			if(v.isEmpty())  continue;
+			c = infimumbound(v);
+			t = new Tuple2<Object, DFAState<Object, ?, Void>>(
+					dfa.getLabelInt(c),
+					dfa.goInt(c));
+			if((g = z.get(t)) == null) {
+				z.put(t, v);
+			} else {
+				z.put(t, g.join(v));
+			}
+		}
+		return z;
+	}
+
 	/**
 	 * 
 	 * @param o
@@ -587,6 +622,16 @@ implements NinaTranslator {
 		}
 	}
 
+	private String escapeHeader(String s) {
+		if(s.equals("'")) {
+			return "\\'";
+		} else if(s.equals("\"")) {
+			return "\\\"";
+		} else {
+			return s;
+		}
+	}
+
 	private ShEnvironment getenv(String p, Quadro q,
 			NinaSubautomata sub) {
 		ShEnvironment env;
@@ -634,6 +679,12 @@ implements NinaTranslator {
 		env.bind("EXEC_LOOKAHEAD_COMMIT",
 				q.getLanguageOption().getLookaheadCommit());
 		env.bind("TAPETYPE", q.getOption("tapetype"));
+		env.bind("NINA_VERSION",  Nina.VERSION);
+		env.bind("NINA_REVISION", Nina.getRevision());
+		env.bind("MATCHER", q.getOptionNvl("matcher", ""));
+		env.bind("COMMENT", q.getOptionNvl("comment", ""));
+		env.bind("COMMENTHEADER", escapeHeader(q.getOptionNvl(
+				"commentHeader", "#")));
 
 		if("prompt".equals(q.getOption("main"))) {
 			env.bind("PUTPROMPT", "true");
