@@ -41,6 +41,7 @@ else
   nullsyml='null'
   streamsym='System.Collections.IEnumerator'
 fi
+sq="'"
 
 awk '{ printf("/// %s\n", $0) }' description
 cat << EOF
@@ -163,6 +164,11 @@ echo "	}"
 [ -n "$ENABLE_LOG" ] && cat << EOF
 EOF
 
+if [ -n "$TYPE" ]; then
+elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
+  echo '	private int __unreadcomment = -2;'
+fi
+
 if [ -n "$ENABLE_BACKTRACK" ]
 then
   echo "	private $CTYPE[] __backtrack = null;"
@@ -195,6 +201,106 @@ cat field
 echo
 
 function _putreadf {
+  if [ -n "$3" ]; then
+  elif [ "$COMMENT" = "shell" ]; then
+    cat << EOF
+	int ReadComment($streamsym stream)
+	{
+		int c;
+
+		c = stream.${readfn}();
+		if(c == ${sq}${COMMENTHEADER}${sq})
+		{
+			while((c = stream.${readfn}()) >= 0)
+			{
+				if(c == '\n')
+				{
+					break;
+				}
+			}
+			return c;
+		}
+		else
+		{
+			return c;
+		}
+	}
+EOF
+  elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
+    cat << EOF
+	int ReadComment($streamsym stream)
+	{
+		int c, state = 0;
+
+		if(__unreadcomment >= -1)
+		{
+			c = __unreadcomment;
+			__unreadcomment = -2;
+			return c;
+		}
+		else if((c = stream.${readfn}()) == '/')
+		{
+			while((c = stream.${readfn}()) >= 0)
+			{
+				switch(state)
+				{
+				case 0:
+					if(c == '*')
+					{
+						state = 100;
+					}
+					else if(c == '/')
+					{
+						state = 200;
+					}
+					else
+					{
+						__unreadcomment = c;
+						return '/';
+					}
+					break;
+				case 100:
+					if(c == '*')
+					{
+						state = 101;
+					}
+					break;
+				case 101:
+					if(c == '/')
+					{
+						return ' ';
+					}
+					else
+					{
+						state = 100;
+					}
+					break;
+				case 200:
+					if(c == '\n')
+					{
+						return '\n';
+					}
+					break;
+				}
+			}
+			__unreadcomment = -1;
+			return '/';
+		}
+		else
+		{
+			return c;
+		}
+	}
+EOF
+  else
+    cat << EOF
+	int ReadComment($streamsym stream)
+	{
+		return stream.${readfn}();
+	}
+EOF
+  fi
+
   [ -n "$ENABLE_INDENT" ] || echo "	$2 $1($streamsym stream)"
   [ -n "$ENABLE_INDENT" ] && echo "	$2 ${1}_1($streamsym stream)"
   echo '	{'
@@ -204,7 +310,7 @@ function _putreadf {
     cat << EOF
 		int c;
 
-		c = stream.${readfn}();
+		c = ReadComment(stream);
 		return c < 0 ? NinaEOF : c;
 EOF
   else
@@ -563,6 +669,8 @@ then
 	private void __Copy_lookahead(int p)
 	{
 		$CTYPE[] a;
+		$CTYPE[] lookaheadOrg = __lookahead;
+		int lookaheadLength = __lookahead != null ? __lookahead.Length : __lookahead_ptr;
 
 		if(__lookahead_ptr > 0)
 		{
@@ -571,19 +679,15 @@ then
 			__lookahead = a;
 		}
 
-		if(__lookahead == null)
-		{
-			a = new $CTYPE[__lookaheadw_ptr];
-		}
-		else if(__lookaheadw_ptr < __lookahead.Length)
-		{
-			a = __lookahead;
-		}
-		else
-		{
-			a = new $CTYPE[__lookaheadw_ptr];
-		}
+		a = new $CTYPE[__lookaheadw_ptr + (lookaheadLength - __lookahead_ptr)];
 		System.Array.Copy(__lookaheadw, a, __lookaheadw_ptr);
+		if(lookaheadOrg != null) {
+			System.Array.Copy(lookaheadOrg,
+					__lookahead_ptr,
+					a,
+					__lookaheadw_ptr,
+					lookaheadLength - __lookahead_ptr);
+		}
 		__lookahead = a;
 		__lookahead_ptr = p;
 		__lookaheadw = null;
