@@ -20,6 +20,7 @@ int='$int'
 num='$num'
 indent='$indent'
 indentBefore='$indentBefore'
+sq="'"
 
 echo '/**'
 awk '{ printf(" * %s\n", $0) }' description
@@ -85,6 +86,11 @@ EOF
 	this.__indent_unread = null;
 EOF
 
+if [ -n "$TYPE" ]; then
+elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
+  echo '	this.__unreadcomment = -2;'
+fi
+
 for i in $SUBAUTOMATA
 do
   ch1=`replace_strange_char $i`
@@ -132,10 +138,10 @@ ${CLASSNAME}.prototype.${1} = function(rd) {
 		c = this.__indent_unread;
 		this.__indent_unread = null;
 	} else if(!this.__is_line_begin) {
-		c = rd();
+		c = this._readComment(rd);
 	} else {
 		this.$indent = 0;
-		while((c = rd()) == ' '.charCodeAt(0) || c == '\t'.charCodeAt(0)) {
+		while((c = this._readComment(rd)) == ' '.charCodeAt(0) || c == '\t'.charCodeAt(0)) {
 			this.$indent++;
 		}
 		this.__indent_unread = c;
@@ -151,7 +157,7 @@ EOF
   else
     cat << EOF
 ${CLASSNAME}.prototype.${1} = function(rd) {
-	return rd();
+	return this._readComment(rd);
 }
 EOF
   fi
@@ -160,8 +166,81 @@ EOF
 cat field
 if [ -z "$LEXER" ]; then
   _putreadf _read1l
+  if [ "$COMMENT" = "shell" ]; then
+    cat << EOF
+${CLASSNAME}.prototype._readComment = function(rd) {
+	var c;
+
+	c = rd();
+	if(c === ${sq}${COMMENTHEADER}${sq}.charCodeAt(0)) {
+		while((c = rd()) !== null) {
+			if(c === '\n'.charCodeAt(0)) {
+				break;
+			}
+		}
+		return c;
+	} else {
+		return c;
+	}
+};
+EOF
+  elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
+    cat << EOF
+${CLASSNAME}.prototype._readComment = function(rd) {
+	var c, state = 0;
+
+	if(this.__unreadcomment === null || this.__unreadcomment >= 0) {
+		c = this.__unreadcomment;
+		this.__unreadcomment = -2;
+		return c;
+	} else if((c = rd()) == '/'.charCodeAt(0)) {
+		while((c = rd()) >= 0) {
+			switch(state) {
+			case 0:
+				if(c == '*'.charCodeAt(0)) {
+					state = 100;
+				} else if(c == '/'.charCodeAt(0)) {
+					state = 200;
+				} else {
+					this.__unreadcomment = c;
+					return '/'.charCodeAt(0);
+				}
+				break;
+			case 100:
+				if(c == '*'.charCodeAt(0)) {
+					state = 101;
+				}
+				break;
+			case 101:
+				if(c == '/'.charCodeAt(0)) {
+					return ' '.charCodeAt(0);
+				} else {
+					state = 100;
+				}
+				break;
+			case 200:
+				if(c == '\n'.charCodeAt(0)) {
+					return '\n'.charCodeAt(0);
+				}
+				break;
+			}
+		}
+		this.__unreadcomment = null;
+		return '/'.charCodeAt(0);
+	} else {
+		return c;
+	}
+};
+EOF
+  else
+    cat << EOF
+${CLASSNAME}.prototype._readComment = function(rd) {
+	return rd();
+};
+EOF
+  fi
   cat << EOF
-	
+
 ${CLASSNAME}.prototype._read1 = function(stream) {
 	var c;
 
@@ -191,6 +270,10 @@ else
   _putreadf _read1l
   nina_template.nfa.js.sub2.sh "$LEXER" "$LEXER" "$CLASSNAME" "$LCTYPE"
   cat << EOF
+
+${CLASSNAME}.prototype._readComment = function(rd) {
+	return rd();
+}
 	
 ${CLASSNAME}.prototype._read1ul = function(stream) {
 	var c;
@@ -365,27 +448,28 @@ ${CLASSNAME}.prototype._f_LOOKAHEAD = function(c) {
 }
 
 ${CLASSNAME}.prototype.__copy_lookahead = function(p) {
-	var a, k;
+	var a,
+		k,
+		lookaheadOrg = this.__lookahead,
+		lookaheadLength = this.__lookahead != null ? this.__lookahead.length : this.__lookahead_ptr;
 
 	if(this.__lookahead_ptr > 0) {
 		a = [];
 		for(k = 0; k < a.length; k++) {
 			a[k] = this.__lookaheadw[k + this.__lookahead_ptr];
 		}
-		__lookahead = a;
-	}
-
-	if(this.__lookahead == null) {
-		a = [];
-	} else if(this.__lookaheadw_ptr < this.__lookahead.length - this.__lookahead_ptr) {
-		a = this.__lookahead;
-	} else {
-		a = [];
+		this.__lookahead = a;
 	}
 
+	a = [];
 	for(k = 0; k < this.__lookaheadw_ptr; k++) {
 		a[k] = this.__lookaheadw[k];
 	}
+	if(lookaheadOrg) {
+		for(k = 0; k < lookaheadLength - this.__lookahead_ptr; k++) {
+			a[this.__lookaheadw_ptr + k] = lookaheadOrg[this.__lookahead_ptr + k];
+		}
+	}
 	this.__lookahead = a;
 	this.__lookahead_ptr = p;
 	this.__lookaheadw = null;
@@ -415,6 +499,10 @@ ${CLASSNAME}.prototype._f_LOOKAHEAD_MARK = function() {
 	this.__lookahead_mark = this.__lookaheadw_ptr;
 }
 
+${CLASSNAME}.prototype._f_LOOKAHEAD_MARK_INIT = function() {
+	this.__lookahead_mark = 0;
+}
+
 EOF
 
 [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
@@ -748,8 +836,19 @@ EOF
 EOF
 cat << EOF
 			} else if(c == null) {
+EOF
+[ -n "$MATCHER" ] || cat << EOF
 				if(!b)  throw "${TOKENERROR}";
-				st[0] = NINA_ACCEPT;  return null;
+EOF
+[ -n "$MATCHER" ] && cat << EOF
+				if(!b) {
+					st[0] = NINA_FAIL;
+					return null;
+				}
+EOF
+cat << EOF
+				st[0] = NINA_ACCEPT;
+				return null;
 			} else {
 				this.__logprint("match failed: ", c);
 				this.__puttrace();
@@ -837,7 +936,10 @@ ${CLASSNAME}.prototype.parse_generic = function(rd, entry) {
 					while((this.__state = this.getdeadstate()) < 0) {
 						if((b = this.execfinally()) != null)  break ot;
 						if(this.__slen-- <= 1) {
-							throw new ${CLASSNAME}_TokenException();
+EOF
+  [ -n "$MATCHER" ] || echo "							throw new ${CLASSNAME}_TokenException();"
+  [ -n "$MATCHER" ] && echo '							return false;'
+  cat << EOF
 						}
 					}
 					c = ${CLASSNAME}.__SKIP__;
@@ -870,7 +972,9 @@ ${CLASSNAME}.prototype.parse_generic = function(rd, entry) {
 			}
 			b = true;
 		}
-		if(!b)  throw new ${CLASSNAME}_TokenException();
+EOF
+  [ -n "$MATCHER" ] || echo "		if(!b)  throw new ${CLASSNAME}_TokenException();"
+  cat << EOF
 		return b;
 	} finally {
 		this.__logclose();
